"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const parser_1 = __importDefault(require("./parser"));
const serialport_1 = __importDefault(require("serialport"));
const util_1 = __importDefault(require("util"));
const misc_1 = require("./constants/misc");
const debug = require('debug')('prgx:channel');
class Channel {
    constructor(portPath) {
        this.portPath = portPath;
        this.tx = new serialport_1.default(portPath, { autoOpen: false, baudRate: misc_1.BAUDRATE });
        this.rx = this.tx.pipe(new parser_1.default());
        this.timeout = 1000;
    }
    open() {
        return new Promise((resolve, reject) => {
            this.tx.open(err => {
                if (err) {
                    debug(`Unable to open port: ${err.message}`);
                    reject(err);
                }
                else {
                    debug(`Successfully open port ${this.portPath}`);
                    resolve(this);
                }
            });
        });
    }
    close() {
        return new Promise((resolve, reject) => {
            this.tx.close(err => {
                if (err) {
                    debug(`Somthing went wrong closing connection: ${err.message}`);
                    reject(err);
                }
                else {
                    debug(`Channel at port ${this.portPath} closed.`);
                    resolve();
                }
            });
        });
    }
    send(packet) {
        this.write(packet);
        return this.read();
    }
    write(packet) {
        if (!this.tx.write(packet)) {
            this.tx.drain();
        }
    }
    read() {
        const timer = new Promise((_, reject) => setTimeout(() => reject(new Error(`Timeout of channel at port ${this.portPath}.`)), this.timeout));
        const readPromise = new Promise(resolve => {
            this.rx.once('data', (packet) => {
                debug(`Received packet: ${util_1.default.inspect(packet)}`);
                resolve(packet);
            });
        });
        return Promise.race([timer, readPromise]);
    }
}
exports.default = Channel;
