"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const POLYNOMIAL = 0x1021;
const INITIAL_VALUE = 0xffff;
const CRC_ERROR_BYTE = 0xe0;
// Compute crc bytes and returns them in array.
// First element is msb, second is lsb.
function computeAsValue(data) {
    const value = data.reduce((crc, byte) => {
        for (let i = 0; i < 8; i++) {
            const bit = ((byte >> (7 - i)) & 1) == 1;
            const c15 = ((crc >> 15) & 1) == 1;
            crc <<= 1;
            if (c15 !== bit)
                crc ^= POLYNOMIAL;
        }
        return crc;
    }, INITIAL_VALUE);
    return value & 0xffff;
}
exports.computeAsValue = computeAsValue;
function computeAsBytes(data) {
    const value = computeAsValue(data);
    return [(value & 0xff00) >> 8, value & 0x00ff];
}
exports.computeAsBytes = computeAsBytes;
function isCrcErrorByte(byte) {
    return byte === CRC_ERROR_BYTE;
}
exports.isCrcErrorByte = isCrcErrorByte;
