"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commands = __importStar(require("./constants/commands"));
const crc = __importStar(require("./crc"));
const channel_1 = __importDefault(require("./channel"));
const serialport_1 = __importDefault(require("serialport"));
const util_1 = __importDefault(require("util"));
const misc_1 = require("./constants/misc");
const packet_1 = require("./constants/packet");
const debug = require('debug')('prgx:programmer');
class Programmer {
    constructor(portPath) {
        this.portPath = portPath;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            const channels = yield this.openChannels();
            const promises = channels.map(channel => channel
                .send(misc_1.HANDSHAKE_BUFFER)
                .then((response) => {
                if (response.equals(misc_1.ACK_BUFFER)) {
                    debug(`Connection established at port ${channel.portPath}`);
                    return channel;
                }
            })
                .catch(err => {
                debug(`Prepare to close channel due to: ${err.message}`);
                return channel.close();
            }));
            const results = yield Promise.all(promises);
            this.channel = results.filter(Boolean)[0];
            if (!this.channel) {
                throw new Error('Connection failed.');
            }
        });
    }
    flash(address, data) {
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setFlashAddress(address);
            const chunks = this.split(Array.from(data));
            try {
                for (var chunks_1 = __asyncValues(chunks), chunks_1_1; chunks_1_1 = yield chunks_1.next(), !chunks_1_1.done;) {
                    const chunk = chunks_1_1.value;
                    yield this.writeFlash(chunk);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (chunks_1_1 && !chunks_1_1.done && (_a = chunks_1.return)) yield _a.call(chunks_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            // Send last command to force programming of last flash page (see manual for details)
            yield this.writeFlash([]);
        });
    }
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.startApplication();
            yield this.channel.close();
        });
    }
    startApplication() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [commands.START_APPLICATION];
            //TODO(>timeout management): decide what to do if we timeout
            // because it could mean that bootloader didn't receive,
            // startApplication packet.
            yield this.send(data);
        });
    }
    getHardwareData() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [commands.READ_VERSION_BOOTLOADER];
            // We are not interested in checking command byte because it cannot fail
            const [_, minor, major, modelNumber] = yield this.send(data);
            const model = `HP${modelNumber}00`;
            const version = `${major}.${minor}`;
            return { model, version };
        });
    }
    getFirmwareVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [commands.READ_VERSION_FIRMWARE];
            // We are not interested in checking command byte because it cannot fail
            const [_, minor, major] = yield this.send(data);
            return `${major}.${minor}`;
        });
    }
    setFlashAddress(address) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [commands.SET_ADDRESS];
            data.push((address >> 24) & 0xff);
            data.push((address >> 16) & 0xff);
            data.push((address >> 8) & 0xff);
            data.push(address & 0xff);
            const response = yield this.send(data);
            if (response[0] === 0x21) {
                throw new Error(`Address ${address} is out of valid range.`);
            }
        });
    }
    writeFlash(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = [commands.FLASH_WRITE_DATA, ...data];
            const response = yield this.send(payload);
            if (response[0] === 0x22) {
                throw new Error('Unable to flash data');
            }
        });
    }
    openChannels() {
        return __awaiter(this, void 0, void 0, function* () {
            let portPaths = [];
            if (this.portPath) {
                portPaths.push(this.portPath);
            }
            else {
                const ports = yield serialport_1.default.list();
                debug(`Found ports: ${ports.map(port => port.path).join(', ')}`);
                ports.forEach(port => portPaths.push(port.path));
            }
            const channels = portPaths.map(path => new channel_1.default(path));
            const promises = channels.map(channel => channel.open());
            const results = yield Promise.all(promises.map(p => p.catch(e => e)));
            return results.filter(result => result instanceof channel_1.default);
        });
    }
    //NOTE: send does not check if command is failed, instead it leaves
    // this task to the caller. That's why it should provide some
    // way to discern timeout errors from packet errors I think.
    // However, until we manage timeout, we will keep this implementation
    send(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                data = this.appendCrc(data);
                const packet = this.createPacket(data);
                // If response is invalid (i.e. its CRC is invalid) or contains a CRC error
                // (i.e. packet sent from PC had wrong CRC) we immediately resend same packet.
                // Number of attempts should be configurable I think...
                for (let attempt = 0; attempt < 10; attempt++) {
                    debug(`Sending packet: ${util_1.default.inspect(packet)} (attempt #${attempt})`);
                    try {
                        const response = yield this.channel.send(packet).then(response => {
                            debug(`Received response: ${util_1.default.inspect(response)}`);
                            return this.removeEscapeByte(response);
                        });
                        if (this.validateResponse(response) && !crc.isCrcErrorByte(response[0])) {
                            return resolve(response);
                        }
                        else {
                            debug(`Invalid response received: ${util_1.default.inspect(Buffer.from(response))}.`);
                        }
                    }
                    catch (_a) {
                        debug('Timeout occured.');
                    }
                }
                return reject(new Error('Maximum number of attempts reached.'));
            }));
        });
    }
    appendCrc(data) {
        const crcBytes = crc.computeAsBytes(data);
        return data.concat(crcBytes);
    }
    createPacket(data) {
        const payload = data.reduce((acc, byte) => {
            if ([packet_1.STX, packet_1.ETX, packet_1.ESC].includes(byte)) {
                return acc.concat([packet_1.ESC, byte | 0x80]);
            }
            return acc.concat(byte);
        }, []);
        return Buffer.from([packet_1.STX, ...payload, packet_1.ETX]);
    }
    removeEscapeByte(packet) {
        let offset = 0x00;
        return packet.reduce((acc, byte) => {
            if (byte === packet_1.ESC) {
                offset = 0x80;
                return acc;
            }
            acc.push(byte - offset);
            offset = 0x00;
            return acc;
        }, []);
    }
    validateResponse(response) {
        // Last two bytes contain CRC value (res[-2] -> msb, res[-1] -> lsb)
        const expected = response.pop() + (response.pop() << 8);
        const actual = crc.computeAsValue(response);
        return expected === actual;
    }
    *split(data) {
        while (data.length > 0) {
            let chunk = data.splice(0, packet_1.MAX_DATA_LENGTH);
            yield Buffer.from(chunk);
        }
    }
}
exports.default = Programmer;
