"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const packet_1 = require("./constants/packet");
const util_1 = __importDefault(require("util"));
const debug = require('debug')('prgx:parser');
class PacketParser extends stream_1.Transform {
    constructor() {
        super();
        this.buffer = Buffer.alloc(0);
    }
    _transform(chunk, encoding, callback) {
        // Buffer always contains data left over from previous calls,
        // so we append `chunk` to it
        this.buffer = Buffer.concat([this.buffer, chunk]);
        // Get bounds of first packet in the stream
        let stx = this.buffer.indexOf(packet_1.STX);
        let etx = this.buffer.indexOf(packet_1.ETX, stx);
        // We iterate as long as we find a complete packet
        while (stx !== -1 && etx !== -1) {
            // Exclude STX and ETX from payload
            const payload = this.buffer.slice(stx + 1, etx);
            debug(`Parsed payload: ${util_1.default.inspect(payload)}`);
            this.push(payload);
            // Process next packet, if exists
            stx = this.buffer.indexOf(packet_1.STX, etx);
            etx = this.buffer.indexOf(packet_1.ETX, stx);
        }
        // If we exit because of an incomplete packet, we push it to the buffer
        if (stx !== -1) {
            this.buffer = this.buffer.slice(stx);
        }
        else {
            this.buffer = Buffer.alloc(0); // Otherwise we clean the buffer because all data has been processed
        }
        callback();
    }
}
exports.default = PacketParser;
